@@ -31,14 +31,15 @@ import static org.joni.Option.isPosixRegion;
 import org.jcodings.CodeRange;
 import org.jcodings.Encoding;
 import org.jcodings.IntHolder;
-import org.joni.constants.OPCode;
-import org.joni.constants.OPSize;
+import org.joni.constants.internal.OPCode;
+import org.joni.constants.internal.OPSize;
 import org.joni.exception.ErrorMessages;
 import org.joni.exception.InternalException;
 
 class ByteCodeMachine extends StackMachine {
     private static final int INTERRUPT_CHECK_EVERY = 30000;
     int interruptCheckCounter = 0; // we modulos this to occasionally check for interrupts
+    volatile boolean interrupted = false;
 
     private int bestLen;          // return value
     private int s = 0;            // current char
@@ -52,11 +53,15 @@ class ByteCodeMachine extends StackMachine {
     private final int[]code;        // byte code
     private int ip;                 // instruction pointer
 
-    ByteCodeMachine(Regex regex, byte[]bytes, int p, int end) {
-        super(regex, bytes, p, end);
+    ByteCodeMachine(Regex regex, Region region, byte[]bytes, int p, int end) {
+        super(regex, region, bytes, p, end);
         this.code = regex.code;
     }
 
+    public void interrupt() {
+        interrupted = true;
+    }
+
     protected int stkp; // a temporary
     private boolean makeCaptureHistoryTree(CaptureTreeNode node) {
         //CaptureTreeNode child;
@@ -146,33 +151,7 @@ class ByteCodeMachine extends StackMachine {
         return true;
     }
 
-    private void debugMatchBegin() {
-        Config.log.println("match_at: " + "str: " + str + ", end: " + end + ", start: " + sstart + ", sprev: " + sprev);
-        Config.log.println("size: " + (end - str) + ", start offset: " + (sstart - str));
-    }
-
-    private void debugMatchLoop() {
-        Config.log.printf("%4d", (s - str)).print("> \"");
-        int q, i;
-        for (i = 0, q = s; i < 7 && q < end && s >= 0; i++) {
-            int len = enc.length(bytes, q, end);
-            while (len-- > 0) {
-                if (q < end) {
-                    Config.log.print(new String(bytes, q++, 1));
-                }
-            }
-        }
-        String str = q < end ? "...\"" : "\"";
-        q += str.length();
-        Config.log.print(str);
-        for (i = 0; i < 20 - (q - s); i++)
-            Config.log.print(" ");
-        StringBuilder sb = new StringBuilder();
-        new ByteCodePrinter(regex).compiledByteCodeToString(sb, ip);
-        Config.log.println(sb.toString());
-    }
-
-    protected final int matchAt(int _range, int _sstart, int _sprev) throws InterruptedException {
+    protected final int matchAt(int _range, int _sstart, int _sprev, boolean interrupt) throws InterruptedException {
         range = _range;
         sstart = _sstart;
         sprev = _sprev;
@@ -185,14 +164,15 @@ class ByteCodeMachine extends StackMachine {
         bestLen = -1;
         s = _sstart;
         pkeep = _sstart;
-        return enc.isSingleByte() || (msaOptions & Option.CR_7_BIT) != 0 ? executeSb() : execute();
+        return enc.isSingleByte() || (msaOptions & Option.CR_7_BIT) != 0 ? executeSb(interrupt) : execute(interrupt);
     }
 
-    private final int execute() throws InterruptedException {
+    private final int execute(boolean interrupt) throws InterruptedException {
         Thread currentThread = Thread.currentThread();
         final int[]code = this.code;
         while (true) {
-            if (interruptCheckCounter++ % INTERRUPT_CHECK_EVERY == 0 && currentThread.isInterrupted()) {
+            if (interrupted ||
+                    (interrupt && interruptCheckCounter++ % INTERRUPT_CHECK_EVERY == 0 && currentThread.isInterrupted())) {
                 currentThread.interrupted();
                 throw new InterruptedException();
             }
@@ -323,11 +303,12 @@ class ByteCodeMachine extends StackMachine {
         } // main while
     }
 
-    private final int executeSb() throws InterruptedException {
+    private final int executeSb(boolean interrupt) throws InterruptedException {
         Thread currentThread = Thread.currentThread();
         final int[]code = this.code;
         while (true) {
-            if (interruptCheckCounter++ % INTERRUPT_CHECK_EVERY == 0 && currentThread.isInterrupted()) {
+            if (interrupted ||
+                    (interrupt && interruptCheckCounter++ % INTERRUPT_CHECK_EVERY == 0 && currentThread.isInterrupted())) {
                 currentThread.interrupted();
                 throw new InterruptedException();
             }
@@ -384,8 +365,8 @@ class ByteCodeMachine extends StackMachine {
 
                 case OPCode.BEGIN_BUF:                  opBeginBuf();              continue;
                 case OPCode.END_BUF:                    opEndBuf();                continue;
-                case OPCode.BEGIN_LINE:                 opBeginLine();             continue;
-                case OPCode.END_LINE:                   opEndLine();               continue;
+                case OPCode.BEGIN_LINE:                 opBeginLineSb();           continue;
+                case OPCode.END_LINE:                   opEndLineSb();             continue;
                 case OPCode.SEMI_END_BUF:               opSemiEndBuf();            continue;
                 case OPCode.BEGIN_POSITION:             opBeginPosition();         continue;
 
@@ -501,21 +482,11 @@ class ByteCodeMachine extends StackMachine {
             }
         } else {
             Region region = msaRegion;
-            if (Config.USE_POSIX_API_REGION_OPTION) {
-                if (!isPosixRegion(regex.options)) {
-                    if (region != null) {
-                        region.clear();
-                    } else {
-                        msaBegin = msaEnd = 0;
-                    }
-                }
+            if (region != null) {
+                region.clear();
             } else {
-                if (region != null) {
-                    region.clear();
-                } else {
-                    msaBegin = msaEnd = 0;
-                }
-            } // USE_POSIX_REGION_OPTION
+                msaBegin = msaEnd = 0;
+            }
         }
         // end_best_len:
         /* default behavior: return first-matching result. */
@@ -1272,6 +1243,16 @@ class ByteCodeMachine extends StackMachine {
         opFail();
     }
 
+    private void opBeginLineSb() {
+        if (s == str) {
+            if (isNotBol(msaOptions)) opFail();
+            return;
+        } else if (bytes[sprev] == Encoding.NEW_LINE && s != end) {
+            return;
+        }
+        opFail();
+    }
+
     private void opEndLine()  {
         if (s == end) {
             if (Config.USE_NEWLINE_AT_END_OF_STRING_HAS_EMPTY_LINE) {
@@ -1289,6 +1270,23 @@ class ByteCodeMachine extends StackMachine {
         opFail();
     }
 
+    private void opEndLineSb()  {
+        if (s == end) {
+            if (Config.USE_NEWLINE_AT_END_OF_STRING_HAS_EMPTY_LINE) {
+                if (str == end || bytes[sprev] != Encoding.NEW_LINE) {
+                    if (isNotEol(msaOptions)) opFail();
+                }
+                return;
+            } else {
+                if (isNotEol(msaOptions)) opFail();
+                return;
+            }
+        } else if (bytes[s] == Encoding.NEW_LINE || (Config.USE_CRNL_AS_LINE_TERMINATOR && enc.isMbcCrnl(bytes, s, end))) {
+            return;
+        }
+        opFail();
+    }
+
     private void opSemiEndBuf() {
         if (s == end) {
             if (Config.USE_NEWLINE_AT_END_OF_STRING_HAS_EMPTY_LINE) {
@@ -1932,4 +1930,30 @@ class ByteCodeMachine extends StackMachine {
     private int finish() {
         return bestLen;
     }
+
+    private void debugMatchBegin() {
+        Config.log.println("match_at: " + "str: " + str + ", end: " + end + ", start: " + sstart + ", sprev: " + sprev);
+        Config.log.println("size: " + (end - str) + ", start offset: " + (sstart - str));
+    }
+
+    private void debugMatchLoop() {
+        Config.log.printf("%4d", (s - str)).print("> \"");
+        int q, i;
+        for (i = 0, q = s; i < 7 && q < end && s >= 0; i++) {
+            int len = enc.length(bytes, q, end);
+            while (len-- > 0) {
+                if (q < end) {
+                    Config.log.print(new String(bytes, q++, 1));
+                }
+            }
+        }
+        String str = q < end ? "...\"" : "\"";
+        q += str.length();
+        Config.log.print(str);
+        for (i = 0; i < 20 - (q - s); i++)
+            Config.log.print(" ");
+        StringBuilder sb = new StringBuilder();
+        new ByteCodePrinter(regex).compiledByteCodeToString(sb, ip);
+        Config.log.println(sb.toString());
+    }
 }
