@@ -27,19 +27,22 @@ import static org.joni.ast.QuantifierNode.isRepeatInfinite;
 import org.jcodings.Ptr;
 import org.jcodings.constants.CharacterType;
 import org.jcodings.exception.CharacterPropertyException;
+import org.jcodings.exception.EncodingError;
 import org.joni.ast.QuantifierNode;
-import org.joni.constants.AnchorType;
 import org.joni.constants.MetaChar;
-import org.joni.constants.TokenType;
+import org.joni.constants.internal.AnchorType;
+import org.joni.constants.internal.TokenType;
 import org.joni.exception.ErrorMessages;
 
 class Lexer extends ScannerSupport {
+    protected final Regex regex;
     protected final ScanEnvironment env;
     protected final Syntax syntax;              // fast access to syntax
     protected final Token token = new Token();  // current token
 
     protected Lexer(Regex regex, Syntax syntax, byte[]bytes, int p, int end, WarnCallback warnings) {
         super(regex.enc, bytes, p, end);
+        this.regex = regex;
         this.env = new ScanEnvironment(regex, syntax, warnings);
         this.syntax = env.syntax;
     }
@@ -944,7 +947,7 @@ class Lexer extends ScannerSupport {
             token.setBackrefNum(1);
             token.setBackrefRef1(backNum);
         } else {
-            NameEntry e = env.reg.nameToGroupNumbers(bytes, last, nameEnd);
+            NameEntry e = regex.nameToGroupNumbers(bytes, last, nameEnd);
             if (e == null) newValueException(UNDEFINED_NAME_REFERENCE, last, nameEnd);
 
             if (syntax.strictCheckBackref()) {
@@ -1009,6 +1012,7 @@ class Lexer extends ScannerSupport {
     }
 
     protected final void fetchToken() {
+        int src = p;
         // mark(); // out
         start:
         while(true) {
@@ -1236,8 +1240,9 @@ class Lexer extends ScannerSupport {
                         if (syntax.opBracketCC()) token.type = TokenType.CC_OPEN;
                         break;
                     case ']':
-                        //if (*src > env->pattern)   /* /].../ is allowed. */
-                        //CLOSE_BRACKET_WITHOUT_ESC_WARN(env, (UChar* )"]");
+                        if (src > getBegin()) { /* /].../ is allowed. */
+                            env.closeBracketWithoutEscapeWarn("]");
+                        }
                         break;
                     case '#':
                         if (Option.isExtend(env.option)) {
@@ -1304,7 +1309,7 @@ class Lexer extends ScannerSupport {
             if (c == '}') {
                 return enc.propertyNameToCType(bytes, _p, last);
             } else if (c == '(' || c == ')' || c == '{' || c == '|') {
-                throw new CharacterPropertyException(ERR_INVALID_CHAR_PROPERTY_NAME, bytes, _p, last);
+                throw new CharacterPropertyException(EncodingError.ERR_INVALID_CHAR_PROPERTY_NAME, bytes, _p, last);
             }
         }
         newInternalException(PARSER_BUG);
@@ -1316,7 +1321,7 @@ class Lexer extends ScannerSupport {
     }
 
     protected final void syntaxWarn(String message) {
-        if (Config.USE_WARN) {
+        if (env.warnings != WarnCallback.NONE) {
             env.warnings.warn(message + ": /" + new String(bytes, getBegin(), getEnd()) + "/");
         }
     }
