@@ -45,19 +45,17 @@ import org.joni.ast.EncloseNode;
 import org.joni.ast.Node;
 import org.joni.ast.QuantifierNode;
 import org.joni.ast.StringNode;
-import org.joni.constants.AnchorType;
-import org.joni.constants.EncloseType;
-import org.joni.constants.NodeType;
-import org.joni.constants.TokenType;
+import org.joni.constants.internal.AnchorType;
+import org.joni.constants.internal.EncloseType;
+import org.joni.constants.internal.NodeType;
+import org.joni.constants.internal.TokenType;
 
 class Parser extends Lexer {
-    protected final Regex regex;
     protected int returnCode; // return code used by parser methods (they itself return parsed nodes)
                               // this approach will not affect recursive calls
 
     protected Parser(Regex regex, Syntax syntax, byte[]bytes, int p, int end, WarnCallback warnings) {
         super(regex, syntax, bytes, p, end, warnings);
-        this.regex = regex;
     }
 
     private static final int POSIX_BRACKET_NAME_MIN_LEN            = 4;
