@@ -342,5 +342,11 @@ public class TestU8 extends Test {
         x3s("\\(((?:[^(]|\\g<0>)*)\\)", "(abc)(abc)", 1, 4, 1);
         x3s("\\(((?:[^(]|\\g<0>)*)\\)", "((abc)(abc))", 1, 11, 1);
         x3s("\\(((?:[^(]|(\\g<0>))*)\\)", "((abc)(abc))", 6, 11, 2);
+
+        x2s("^.+$", "a\n", 0, 1);
+        x2s("^.+$", "\na\n", 1, 2);
+        ns("^.+$", "\n");
+
+        ns("💌", "aa");
     }
 }
